#
# Copyright (C) Orange
#
# This software is distributed under the terms and conditions of the 'MIT'
# license which can be found in the file 'LICENSE.md' in this package distribution

import time
import sys


def use_existing_network_connection():
    print('Using existing network connection')


def get_mac():
    import uuid
    return ''.join(['{:02x}'.format((uuid.getnode() >> ele) & 0xff) for ele in range(0, 8*6, 8)][::-1]).upper()


CONN_TIMEOUT = 20


def wifi_connect(ssid, password):
    from network import WLAN, STA_IF

    sta_if = WLAN(STA_IF)
    sta_if.active(True)
    start_time = time.time()
    while True:
        print('Connecting to network...')
        try:
            sta_if.connect(ssid, password)
            time.sleep_ms(3000)
            if sta_if.isconnected():
                print("WiFi connected successfully.")
                print('Network config:', sta_if.ifconfig())
                break
            elif time.time() - start_time > CONN_TIMEOUT:
                print("[ERROR] Wi-Fi not connected. Stopped.")
                sys.exit()
        except OSError:
            print("[ERROR] Wi-Fi not connected. Stopped.")
            sys.exit()


def get_esp_mac():
    from network import WLAN
    import binascii
    return binascii.hexlify(WLAN().config('mac')).decode('ascii').upper()


def pycom_wifi_connect(ssid, password, hostname):
    from network import WLAN

    wlan = WLAN(mode=WLAN.STA)
    wlan.hostname(hostname)
    start_time = time.time()
    while True:
        print("Trying to connect...")
        wlan.connect(ssid=ssid, auth=(WLAN.WPA2, password))
        time.sleep_ms(3000)
        if wlan.isconnected():
            print("WiFi connected successfully.")
            print('IPs:', wlan.ifconfig(), 'Channel:', wlan.channel())
            break
        elif time.time() - start_time > CONN_TIMEOUT:
            print("[ERROR] Wi-Fi not connected. Stopped.")
            sys.exit()


def get_pycom_mac():
    from network import WLAN
    import binascii
    return binascii.hexlify(WLAN().mac()[0]).decode('ascii').upper()


def lte_connect(pin):
    from network import LTE

    def is_sim_waiting_for_pin():
        if lte.send_at_cmd('AT+CPIN?').strip() == '+CPIN: SIM PIN\r\n\r\nOK':
            return True
        else:
            return False

    lte = LTE()
    time.sleep(2)

    if is_sim_waiting_for_pin():
        print("PIN", (lte.send_at_cmd('AT+CPIN="%s"' % pin)).strip())
    else:
        print("PIN PRESENT: OK")

    lte.attach()
    print("Attaching... ", end='')
    while not lte.isattached():
        time.sleep(1)
    print("attached!")

    lte.connect()
    print("Connecting... ", end='')
    while not lte.isconnected():
        time.sleep(1)
    print("connected!")


def get_pycom_imei():
    from network import LTE
    return LTE().imei()



#
# Copyright (C) Orange
#
# This software is distributed under the terms and conditions of the 'MIT'
# license which can be found in the file 'LICENSE.md' in this package distribution

import sys
import LiveObjects


class BoardsInterface:

    DEFAULT_CARRIER = 1
    EXISTING_NETWORK = 2
    WIFI = 3
    LTE = 4

    PYTHON = 1
    MICROPYTHON = 2

    @staticmethod
    def create_credentials(net_type):
        return LiveObjects.Credentials(net_type)

    def get_apikey(self):
        return self._credentials.get_apikey()

    def get_client_id(self):
        pass

    def get_lang_str(self):
        lang_dict = {BoardsInterface.PYTHON: 'urn:lo:nsid:Python',
                     BoardsInterface.MICROPYTHON: 'urn:lo:nsid:microPython'}
        return lang_dict[self._lang_id]

    def get_lang_id(self):
        return self._lang_id

    def get_security_level(self):
        pass

    def get_store_cert_filename(self):
        pass

    def check_network_capabilities(self, net_type):
        if net_type not in self._carrier_capability:
            print('Carrier not supported.')
            sys.exit()

    def connect(self):
        pass

    def network_disconnect(self):
        pass


class LoPy(BoardsInterface):
    def __init__(self, net_type):
        self._lang_id = BoardsInterface.MICROPYTHON
        self._net_type = BoardsInterface.WIFI if net_type == BoardsInterface.DEFAULT_CARRIER else net_type
        self._carrier_capability = (BoardsInterface.WIFI,)
        self._wifi_tls_capability = False
        self._credentials = super().create_credentials(self._net_type)
        self._hostname = 'LoPy'

    def connect(self):
        super().check_network_capabilities(self._net_type)
        if self._net_type == BoardsInterface.WIFI:
            from LiveObjects.services import pycom_wifi_connect
            pycom_wifi_connect(self._credentials.get_creds()['ssid'], self._credentials.get_creds()['password'],
                               self._hostname)

    def get_security_level(self):
        if self._net_type == BoardsInterface.WIFI:
            return LiveObjects.SSL if self._wifi_tls_capability else LiveObjects.NONE

    def get_client_id(self):
        from LiveObjects.services import get_pycom_mac
        return self.get_lang_str() + ':' + get_pycom_mac()


class GPy(BoardsInterface):
    def __init__(self, net_type):
        self._lang_id = BoardsInterface.MICROPYTHON
        self._net_type = BoardsInterface.WIFI if net_type == BoardsInterface.DEFAULT_CARRIER else net_type
        self._carrier_capability = (BoardsInterface.WIFI, BoardsInterface.LTE)
        self._wifi_tls_capability = False
        self._lte_tls_capability = False
        self._credentials = super().create_credentials(self._net_type)
        self._hostname = 'GPy'

    def connect(self):
        super().check_network_capabilities(self._net_type)
        if self._net_type == BoardsInterface.WIFI:
            from LiveObjects.services import pycom_wifi_connect
            pycom_wifi_connect(self._credentials.get_creds()['ssid'], self._credentials.get_creds()['password'],
                               self._hostname)
        elif self._net_type == BoardsInterface.LTE:
            from LiveObjects.services import lte_connect
            lte_connect(self._credentials.get_creds()['pin'])

    def get_security_level(self):
        if self._net_type == BoardsInterface.WIFI:
            return LiveObjects.SSL if self._wifi_tls_capability else LiveObjects.NONE
        elif self._net_type == BoardsInterface.LTE:
            return LiveObjects.SSL if self._lte_tls_capability else LiveObjects.NONE

    def get_client_id(self):
        if self._net_type == BoardsInterface.WIFI:
            from LiveObjects.services import get_pycom_mac
            return self.get_lang_str() + ':' + get_pycom_mac()
        elif self._net_type == BoardsInterface.LTE:
            from LiveObjects.services import get_pycom_imei
            return self.get_lang_str() + ':' + get_pycom_imei()


class Esp8266(BoardsInterface):
    def __init__(self, net_type):
        self._lang_id = BoardsInterface.MICROPYTHON
        self._net_type = BoardsInterface.WIFI if net_type == BoardsInterface.DEFAULT_CARRIER else net_type
        self._carrier_capability = (BoardsInterface.WIFI,)
        self._wifi_tls_capability = False
        self._credentials = super().create_credentials(self._net_type)

    def connect(self):
        from LiveObjects.services import wifi_connect
        super().check_network_capabilities(self._net_type)
        wifi_connect(self._credentials.get_creds()['ssid'], self._credentials.get_creds()['password'])

    def get_security_level(self):
        return LiveObjects.SSL if self._wifi_tls_capability else LiveObjects.NONE

    def get_client_id(self):
        from LiveObjects.services import get_esp_mac
        return self.get_lang_str() + ':' + get_esp_mac()


class Win32(BoardsInterface):
    def __init__(self, net_type):
        self._lang_id = BoardsInterface.PYTHON
        self._net_type = BoardsInterface.EXISTING_NETWORK if net_type == BoardsInterface.DEFAULT_CARRIER else net_type
        self._carrier_capability = (BoardsInterface.EXISTING_NETWORK,)
        self._existing_network_tls_capability = True
        self._credentials = super().create_credentials(self._net_type)

    def connect(self):
        from LiveObjects.services import use_existing_network_connection
        super().check_network_capabilities(self._net_type)
        use_existing_network_connection()

    def get_security_level(self):
        return LiveObjects.SSL if self._existing_network_tls_capability else LiveObjects.NONE

    def get_store_cert_filename(self):
        try:
            import certifi
            return certifi.where()
        except ImportError:
            print("[ERROR] U have missing library 'python-certifi-win32'")
            sys.exit()

    def get_client_id(self):
        from LiveObjects.services import get_mac
        return self.get_lang_str() + ':' + get_mac()


class Esp32(BoardsInterface):
    def __init__(self, net_type):
        self._lang_id = BoardsInterface.MICROPYTHON
        self._net_type = BoardsInterface.WIFI if net_type == BoardsInterface.DEFAULT_CARRIER else net_type
        self._carrier_capability = (BoardsInterface.WIFI,)
        self._wifi_tls_capability = True
        self._credentials = super().create_credentials(self._net_type)

    def connect(self):
        from LiveObjects.services import wifi_connect
        super().check_network_capabilities(self._net_type)
        wifi_connect(self._credentials.get_creds()['ssid'], self._credentials.get_creds()['password'])

    def get_security_level(self):
        return LiveObjects.SSL if self._wifi_tls_capability else LiveObjects.NONE

    def get_client_id(self):
        from LiveObjects.services import get_esp_mac
        return self.get_lang_str() + ':' + get_esp_mac()


class Linux(BoardsInterface):
    def __init__(self, net_type):
        self._lang_id = BoardsInterface.PYTHON
        self._net_type = BoardsInterface.EXISTING_NETWORK if net_type == BoardsInterface.DEFAULT_CARRIER else net_type
        self._carrier_capability = (BoardsInterface.EXISTING_NETWORK,)
        self._existing_network_tls_capability = True
        self._credentials = super().create_credentials(self._net_type)
        self._cert_store_filename = "/etc/ssl/certs/ca-certificates.crt"

    def connect(self):
        from LiveObjects.services import use_existing_network_connection
        super().check_network_capabilities(self._net_type)
        use_existing_network_connection()

    def get_security_level(self):
        return LiveObjects.SSL if self._existing_network_tls_capability else LiveObjects.NONE

    def get_store_cert_filename(self):
        return self._cert_store_filename

    def get_client_id(self):
        from LiveObjects.services import get_mac
        return self.get_lang_str() + ':' + get_mac()


class RaspberryPi(Linux):
    pass


def is_raspberrypi():
    try:
        with open('/proc/device-tree/model') as f:
            return f.read().startswith('Raspberry')
    except FileNotFoundError:
        return False


class BoardsFactory:
    def __new__(cls, net_type):
        s = sys.platform
        sn = s[0].upper() + s[1:]   # capitalize first letter
        if sn == 'Linux':
            sn = 'RaspberryPi' if is_raspberrypi() else 'Linux'
        board = eval(sn)(net_type)  # instance of board w/ net type: WiFi, LTE, etc.
        return board


MAX_DEV_NB = 20


def get_i2c():
    import machine
    typical_gpio = ([22, 23], [5, 4], [22, 21], [23, 18])
    for gpio in typical_gpio:
        scl, sda = gpio
        i2c = None
        try:                    # MicroPython 1.19.1 20220618-v1.19.1
            i2c = machine.SoftI2C(scl=machine.Pin(scl), sda=machine.Pin(sda), freq=100000)
            if i2c.scan() and len(i2c.scan()) < MAX_DEV_NB:
                return i2c
        except ValueError:      # next sda, scl
            pass
        except AttributeError:  # Pycom MicroPython 1.20.2.r6 [v1.11-c5a0a97] on 2021-10-28
            i2c = machine.I2C(0)
            return i2c
        del i2c
    raise RuntimeError("No I2C devices found. Check SDA and SCL lines and add respective GPIO to 'typical_gpio'.")


class SensorVL6180X:
    def __new__(cls):
        try:                # Python@RPi
            import busio
            import adafruit_vl6180x
            import board

            class VL6180X(adafruit_vl6180x.VL6180X):
                def amb_light(self):
                    """Implementing default gain"""
                    return self.read_lux(gain=0x06)     # ALS_GAIN_1

            i2c = busio.I2C(board.SCL, board.SDA)
            return VL6180X(i2c)

        except ImportError:  # microPython
            import vl6180x_micro
            i2c = get_i2c()
            return vl6180x_micro.Sensor(i2c)

        except NotImplementedError:  # if no I2C device
            print("No GPIO present.")
            sys.exit()



#
# Copyright (C) Orange
#
# This software is distributed under the terms and conditions of the 'MIT'
# license which can be found in the file 'LICENSE.md' in this package distribution
import LiveObjects


class Credentials:

    def __init__(self, net_type):

        self._apikey = '<APIKEY>'
        self._net_type = net_type

        if net_type == LiveObjects.BoardsInterface.WIFI:
            self._wifi_ssid = '<WIFI_SSID>'
            self._wifi_password = '<WIFI_PASS>'
        elif net_type == LiveObjects.BoardsInterface.LTE:
            self._pin = '<PIN>'
            self._apn = '<APN>'

    def get_apikey(self):
        return self._apikey

    def get_creds(self):
        if self._net_type == LiveObjects.BoardsInterface.WIFI:
            return {'ssid': self._wifi_ssid, 'password': self._wifi_password}
        elif self._net_type == LiveObjects.BoardsInterface.LTE:
            return {'pin': self._pin, 'apn_name': self._apn}



#
# Copyright (C) Orange
#
# This software is distributed under the terms and conditions of the 'MIT'
# license which can be found in the file 'LICENSE.md' in this package distribution

import sys
import json
import time
import LiveObjects

INT = "i32"
UINT = "u32"
BINARY = "bin"
STRING = "str"
FLOAT = "f64"
INFO = "INFO"
WARNING = "WARNING"
ERROR = "ERROR"

SSL = 8883
NONE = 1883


class LiveObjectsParameter:
    def __init__(self, value, type_, cb=None):
        self.value = value
        self.type = type_
        self.callback = cb


class Connection:
    def __init__(self, debug=True):
        self.__board = LiveObjects.BoardsFactory(net_type=LiveObjects.BoardsInterface.DEFAULT_CARRIER)
        self.mode = self.__board.get_lang_id()

        try:
            if self.mode == LiveObjects.BoardsInterface.PYTHON:
                import paho.mqtt.client as paho
                import os
            elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
                from umqttrobust import MQTTClient
        except ImportError:
            print("[ERROR] U have missing libraries 'Paho-mqtt' (for Python) or 'umqttrobust' (for uPython)")
            sys.exit()

        self.__port = self.__board.get_security_level()
        self.__apiKey = self.__board.get_apikey()
        self.__device_id = self.__board.get_client_id()
        self.__parameters = {}
        self.__server = "mqtt.liveobjects.orange-business.com"
        self.__topic = "dev/data"
        self.__value = "value"
        self.__payload = {self.__value: {}}
        self.__commands = {}
        self.__doLog = debug
        self.quit = False

        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            self.__mqtt = paho.Client(self.__device_id)
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            self.ssl = self.__port == SSL
            self.__mqtt = MQTTClient(self.__device_id, self.__server, self.__port, "json+device",
                                     self.__apiKey, 0, self.ssl, {'server_hostname': self.__server})

    def loop(self):
        if self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            self.__mqtt.check_msg()

    def __on_message(self, client="", userdata="", msg=""):
        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            if msg.topic == "dev/cfg/upd":
                self.__parameter_manager(msg)
            elif msg.topic == "dev/cmd":
                self.__command_manager(msg)
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            if client == b"dev/cfg/upd":
                self.__parameter_manager(userdata)
            elif client == b"dev/cmd":
                self.__command_manager(userdata)

    def __on_connect(self, client="", userdata="", flags="", rc=""):
        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            if rc == 0:
                self.output_debug(INFO, "Connected!")
                if len(self.__commands) > 0:
                    self.output_debug(INFO, "Subscribing commands")
                    self.__mqtt.subscribe("dev/cmd")
                if len(self.__parameters) > 0:
                    self.output_debug(INFO, "Subscribing parameters")
                    self.__mqtt.subscribe("dev/cfg/upd")
                    self.__send_config()
            else:
                self.output_debug(ERROR, "Check your api key")
                self.quit = True
                sys.exit()
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            self.output_debug(INFO, "Connected, sending config")
            if len(self.__commands) > 0:
                self.output_debug(INFO, "Subscribing commands")
                self.__mqtt.subscribe(b"dev/cmd")
            if len(self.__parameters) > 0:
                self.output_debug(INFO, "Subscribing parameters")
                self.__mqtt.subscribe(b"dev/cfg/upd")
                self.__send_config()

    def connect(self):
        self.__board.connect()
        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            self.__mqtt.username_pw_set("json+device", self.__apiKey)
            self.__mqtt.on_connect = self.__on_connect
            self.__mqtt.on_message = self.__on_message
            if self.__port == SSL:
                self.__mqtt.tls_set(self.__board.get_store_cert_filename())
            self.__mqtt.connect(self.__server, self.__port, 60)
            self.__mqtt.loop_start()
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            self.__mqtt.set_callback(self.__on_message)
            self.__mqtt.connect()
            time.sleep(1)
            self.__on_connect()

    def disconnect(self):
        self.__mqtt.disconnect()
        self.output_debug(INFO, "Disconnected")

    def output_debug(self, info, *args):
        if self.__doLog:
            print("[", info, "]", end=" ", sep="")
            for arg in args:
                print(arg, end=" ")
            print("")

    def add_command(self, name, cmd):
        self.__commands[name] = cmd

    def __command_manager(self, msg):
        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            msg_dict = json.loads(msg.payload)
            self.output_debug(INFO, "Received message:\n", json.dumps(msg_dict, sort_keys=True, indent=4))
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            msg_dict = json.loads(msg)
            self.output_debug(INFO, "Received message:", json.dumps(msg_dict))
        output_msg = {}
        output_msg["cid"] = msg_dict["cid"]
        response = self.__commands.get(msg_dict["req"], self.__default)(msg_dict["arg"])
        if len(response) > 0:
            output_msg["res"] = response
        self.__publish_message("dev/cmd/res", output_msg)

    def __default(self, req=""):
        self.output_debug(INFO, "Command not found!")
        return {"info": "Command not found"}

    def __send_config(self):
        out_msg = {"cfg": {}}
        for param in self.__parameters:
            out_msg["cfg"][param] = {"t": self.__parameters[param].type, "v": self.__parameters[param].value}
        self.__publish_message("dev/cfg", out_msg)

    def __parameter_manager(self, msg):
        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            self.output_debug(INFO, "Received message: ")
            self.output_debug(INFO, json.loads(msg.payload))
            params = json.loads(msg.payload)
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            self.output_debug(INFO, "Received message: ")
            self.output_debug(INFO, json.loads(msg))
            params = json.loads(msg)

        for param in params["cfg"]:
            if params["cfg"][param]["t"] == "i32":
                self.__parameters[param].type = INT
            elif params["cfg"][param]["t"] == "str":
                self.__parameters[param].type = STRING
            elif params["cfg"][param]["t"] == "bin":
                self.__parameters[param].type = BINARY
            elif params["cfg"][param]["t"] == "u32":
                self.__parameters[param].type = UINT
            elif params["cfg"][param]["t"] == "f64":
                self.__parameters[param].type = FLOAT

            self.__parameters[param].value = params["cfg"][param]["v"]
            if self.__parameters[param].callback is not None:
                self.__parameters[param].callback(param, params["cfg"][param]["v"])
        self.__publish_message("dev/cfg", params)

    def add_parameter(self, name, val, type_, cb=None):
        if type_ == INT:
            val = int(val)
        elif type_ == STRING:
            val = str(val)
        elif type_ == BINARY:
            val = bool(val)
        elif type_ == UINT:
            val = int(val)
        elif type_ == FLOAT:
            val = float(val)
        self.__parameters[name] = LiveObjectsParameter(val, type_, cb)

    def get_parameter(self, name):
        if self.__parameters[name].type == INT:
            return int(self.__parameters[name].value)
        elif self.__parameters[name].type == STRING:
            return str(self.__parameters[name].value)
        elif self.__parameters[name].type == BINARY:
            return bool(self.__parameters[name].value)
        elif self.__parameters[name].type == UINT:
            return int(self.__parameters[name].value)
        elif self.__parameters[name].type == FLOAT:
            return float(self.__parameters[name].value)
        return 0

    def add_to_payload(self, name, val):
        self.add_model("Orange")
        self.__payload[self.__value][name] = val
    
    def set_object_as_payload(self, val):
        self.__payload[self.__value] = val

    def add_model(self, model):
        self.__payload["model"] = model

    def add_tag(self, tag):
        if "tags" not in self.__payload:
            self.__payload["tags"] = []
        self.__payload["tags"].append(tag)

    def add_tags(self, tags):
        if "tags" not in self.__payload:
            self.__payload["tags"] = []
        for tag in tags:
            self.__payload["tags"].append(tag)

    def send_data(self):
        if self.quit:
            sys.exit()
        self.__publish_message("dev/data", self.__payload)
        self.__payload = {}
        self.__payload[self.__value] = {}

    def __publish_message(self, topic, msg):
        self.output_debug(INFO, "Publishing message on topic: ", topic)

        if self.mode == LiveObjects.BoardsInterface.PYTHON:
            self.output_debug(INFO, "\n", json.dumps(msg, sort_keys=True, indent=4))
        elif self.mode == LiveObjects.BoardsInterface.MICROPYTHON:
            self.output_debug(INFO, json.dumps(msg))

        self.__mqtt.publish(topic, json.dumps(msg))



from setuptools import setup
setup(
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",

    name='LiveObjects',
    url='https://github.com/DatavenueLiveObjects/LiveObjects_SDK_for_Python',
    author='Kacper Sawicki, Krzysztof Krześlak, Tomasz Malek',
    #author_email='',

    packages=['LiveObjects'],
    include_package_data=True,
    install_requires=['paho-mqtt'],

    version='2.1.0',
    license='MIT',


    description='This module allows to easy send data to Orange LiveObjects using Python3 and uPython',
)
